#ifndef INC_INA219_H_
#define INC_INA219_H_

#include <stdint.h>

// default 7-bit I2C address 0x21 is valid when A0 pin is connected using 100k resistor to ground.
// If you use different resistor, edit address here or use -D compiler parameter.
#ifndef INA219_I2C_7BIT_ADDRESS
#define INA219_I2C_7BIT_ADDRESS					0x40
#endif

// default resistor value is 100mOhm. If you use different resistor, then edit value here or use
// -D compiler parameter
#ifndef INA219_SHUNT_RESISOTR_VALUE
#define INA219_SHUNT_RESISOTR_VALUE				0.100
#endif

#define INA219_I2C_READ_ADDRESS					((INA219_I2C_7BIT_ADDRESS << 1) | 0x01)
#define INA219_I2C_WRITE_ADDRESS				((INA219_I2C_7BIT_ADDRESS << 1) | 0x00)

typedef enum {
	INA219_Status_Ok = 0,
	INA219_Status_I2CNack = -10,
	INA219_Status_I2CError = -11,
	INA219_Status_I2CTimeout = -12,
	INA219_Status_PacketErrorCheckFailed = -13,
	INA219_Status_NotImplemented = -20,
	INA219_Status_BadArg = -21,
	INA219_Status_InvalidOperation = -22,
	INA219_Status_NoData = -23,
	INA219_Status_NotSupported = -24,
} INA219_Status;

INA219_Status INA219_Init();
INA219_Status INA219_Configure(uint16_t configurationRegister, uint16_t calibration);
INA219_Status INA219_ReadRawVoltage(uint16_t* rawVoltage);
INA219_Status INA219_ReadRawCurrent(uint16_t* rawCurrent);
INA219_Status INA219_ReadVoltage(float* voltage);
INA219_Status INA219_ReadCurrent(float* current);

#endif
