#include "AdcDriver.h"
#include "stm32l5xx_hal.h"
#include "UART.h"
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os2.h"

extern ADC_HandleTypeDef hadc1;
extern osMutexId_t adc_statusHandle;

static int treshold = 10;

#define DIGITAL_FILTER_SIZE 10

static int currentVal = 0;

static int AdcDriver_Measure() {
	HAL_ADC_Start(&hadc1);

	vTaskDelay(1);
	HAL_ADC_PollForConversion(&hadc1, 1);

	int value = (int)HAL_ADC_GetValue(&hadc1);

	if (value > 3900) {
		return 3900;
	} else {
		return value;
	}
}

void AdcDriver_ThreadEntry() {
	while (1) {
		int diff = AdcDriver_Measure() - currentVal;
		int tresholdLocal;

		osMutexAcquire(adc_statusHandle, portMAX_DELAY);
		tresholdLocal = treshold;
		osMutexRelease(adc_statusHandle);

		if (diff < -tresholdLocal || diff > tresholdLocal) {
			UART_PrintString("Potentiometer change. Difference: ");
			UART_PrintFormattedInteger("%d\r\n", diff);

			int digitalFilter = 0;
			for (int i = 0; i < DIGITAL_FILTER_SIZE; i++) {
				digitalFilter += AdcDriver_Measure();
			}

			osMutexAcquire(adc_statusHandle, portMAX_DELAY);
			currentVal = digitalFilter / DIGITAL_FILTER_SIZE;
			osMutexRelease(adc_statusHandle);
		}

		vTaskDelay(7);
	}
}

void AdcDriver_SetPotentiometerChangeThreshold(int value) {
	osMutexAcquire(adc_statusHandle, portMAX_DELAY);
	treshold = value;
	osMutexRelease(adc_statusHandle);
}

float AdcDriver_GetPotentiometerValue() {
	float output;
	osMutexAcquire(adc_statusHandle, portMAX_DELAY);
	output = (float)currentVal / 4096.0;
	osMutexRelease(adc_statusHandle);
	return output;
}
