#include <ButtonDriver.h>
#include <stdint.h>
#include "stm32l5xx_hal.h"
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os2.h"

extern osMutexId_t btn1_statusHandle;
extern osMutexId_t btn2_statusHandle;
extern osMutexId_t btn3_statusHandle;
extern osMutexId_t btn4_statusHandle;

static osMutexId_t* btnMutex[4] = { &btn1_statusHandle, &btn2_statusHandle, &btn3_statusHandle, &btn4_statusHandle };
static TickType_t pressTime[4] = { 0, 0, 0, 0 };
static int isPressed[4] = { 0, 0, 0, 0 };
static int isLongPressed[4] = { 0, 0, 0, 0 };

void ButtonDriver_ThreadEntry(int btnNo, GPIO_TypeDef *btnGpio, uint16_t btnGpioPin) {
	int prevSatus = 0;

	while (1) {
		int status = HAL_GPIO_ReadPin(btnGpio, btnGpioPin);

		TickType_t now = xTaskGetTickCount();

		if (prevSatus == 1 && status == 0) {
			osMutexAcquire(*(btnMutex[btnNo]), portMAX_DELAY);
			isPressed[btnNo] = 1;
			pressTime[btnNo] = now;
			osMutexRelease(*(btnMutex[btnNo]));
		}


		if (prevSatus == 0 && status == 1) {
			if (now - pressTime[btnNo] > 2000) {
				osMutexAcquire(*(btnMutex[btnNo]), portMAX_DELAY);
				isLongPressed[btnNo] = 1;
				osMutexRelease(*(btnMutex[btnNo]));
			}
		}

		prevSatus = status;

		vTaskDelay(15);
	}

}

int ButtonDriver_WasPressed(int btnNo) {
	int output;
	osMutexAcquire(*(btnMutex[btnNo]), portMAX_DELAY);
	output = isPressed[btnNo];
	isPressed[btnNo] = 0;
	osMutexRelease(*(btnMutex[btnNo]));
	return output;
}

int ButtonDriver_WasLongPressed(int btnNo) {
	int output;
	osMutexAcquire(*(btnMutex[btnNo]), portMAX_DELAY);
	output = isLongPressed[btnNo];
	isLongPressed[btnNo] = 0;
	osMutexRelease(*(btnMutex[btnNo]));
	return output;
}
