#include "INA219_PlatformSpecific.h"
#include "INA219.h"

#include <stdio.h>
#include "UART.h"

INA219_Status INA219_Init() {
	return INA219_PlatformSpecific_Init();
}

INA219_Status INA219_Configure(uint16_t configurationRegister, uint16_t calibration) {
	INA219_Status status;

	status = INA219_PlatformSpecific_Write(0x00, (uint8_t*)&configurationRegister, sizeof(configurationRegister));
	if (status) {
		return status;
	}

	status = INA219_PlatformSpecific_Write(0x05, (uint8_t*)&calibration, sizeof(calibration));
	if (status) {
		return status;
	}

	return INA219_Status_Ok;
}

INA219_Status INA219_ReadRawVoltage(uint16_t* rawVoltage) {
	INA219_Status status;

	status = INA219_PlatformSpecific_Read(0x02, (uint8_t*)rawVoltage, sizeof(*rawVoltage));
	if (status) {
		return status;
	}

	/*
	if ((*rawVoltage & 0x0002) == 0) {
		return INA219_Status_NoData;
	}
	*/

	*rawVoltage = *rawVoltage >> 3;

	return INA219_Status_Ok;
}

INA219_Status INA219_ReadRawCurrent(uint16_t* rawCurrent) {
	INA219_Status status;

	status = INA219_PlatformSpecific_Read(0x01, (uint8_t*)rawCurrent, sizeof(*rawCurrent));
	if (status) {
		return status;
	}

	return INA219_Status_Ok;
}

INA219_Status INA219_ReadVoltage(float* voltage) {
	INA219_Status status;
	uint16_t rawVoltage;

	status = INA219_ReadRawVoltage(&rawVoltage);
	if (status) {
		return status;
	}

	*voltage = rawVoltage * 0.004;

	return INA219_Status_Ok;
}

INA219_Status INA219_ReadCurrent(float* current) {
	INA219_Status status;
	uint16_t rawCurrent;

	status = INA219_ReadRawCurrent(&rawCurrent);
	if (status) {
		return status;
	}

	float voltageDrop = (rawCurrent & 0x7FFF) * 0.00001;
	*current = voltageDrop / INA219_SHUNT_RESISOTR_VALUE;

	if (rawCurrent & 0x8000) {
		*current *= -1;
	}

	return INA219_Status_Ok;
}
