#include "INA219Driver.h"
#include "INA219.h"
#include "UART.h"
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os2.h"

static TickType_t lastSampleTime = 0;
static float lastCurrent = 0;
static float lastVoltage = 0;

extern osMutexId_t ina219_sample_accessHandle;

void INA219Driver_ThreadEntry() {
	INA219_Status inaStatus;

	do {
		inaStatus = INA219_Init();
		if (inaStatus) {
			UART_PrintString("Error while initializing INA219\r\n");
		}
	} while (inaStatus);

	while (1) {
		float accumulatedCurrent = 0;
		float accumulatedVoltage = 0;

		int accumulatedCurrentsCount = 0;
		int accumulatedVoltagesCount = 0;

		for (int i = 0; i < 8; i++) {
			float current, voltage;

			inaStatus = INA219_ReadCurrent(&current);
			if (inaStatus) {
				UART_PrintString("Error while reading INA219 current sample. Error code: ");
				UART_PrintFormattedInteger("%d", inaStatus);
				UART_PrintString("\r\n");
				continue;
			}

			accumulatedCurrent += current;
			accumulatedCurrentsCount++;

			inaStatus = INA219_ReadVoltage(&voltage);
			if (inaStatus) {
				UART_PrintString("Error while reading INA219 voltage sample. Error code: ");
				UART_PrintFormattedInteger("%d", inaStatus);
				UART_PrintString("\r\n");
				continue;
			}

			accumulatedVoltage += voltage;
			accumulatedVoltagesCount++;

			vTaskDelay(1);
		}

		if (accumulatedCurrentsCount != 0 && accumulatedVoltagesCount != 0) {
			float current = accumulatedCurrent / accumulatedCurrentsCount;
			float voltage = accumulatedVoltage / accumulatedVoltagesCount;

			osMutexAcquire(ina219_sample_accessHandle, portMAX_DELAY);
			lastCurrent = current;
			lastVoltage = voltage;
			lastSampleTime = xTaskGetTickCount();
			osMutexRelease(ina219_sample_accessHandle);

			// for some time opnly refresh timestamp for prevent display blinking too fast
			for (int i = 0; i < 100; i++) {
				osMutexAcquire(ina219_sample_accessHandle, portMAX_DELAY);
				lastSampleTime = xTaskGetTickCount();
				osMutexRelease(ina219_sample_accessHandle);
				vTaskDelay(3);
			}
		} else {
			vTaskDelay(1);
		}
	}
}

void INA219Driver_GetLastSample(TickType_t *timestamp, float *current, float *voltage) {
	osMutexAcquire(ina219_sample_accessHandle, portMAX_DELAY);
	*current = lastCurrent;
	*voltage = lastVoltage;
	*timestamp = lastSampleTime;
	osMutexRelease(ina219_sample_accessHandle);
}
