#include "INA219.h"
#include "INA219_PlatformSpecific.h"
#include <stdint.h>
#include <stddef.h>
#include "stm32l5xx_hal.h"

extern I2C_HandleTypeDef hi2c4;

INA219_Status INA219_PlatformSpecific_Init() {
	return INA219_Status_Ok;
}

INA219_Status INA219_PlatformSpecific_Deinit() {
	return INA219_Status_Ok;
}

INA219_Status INA219_PlatformSpecific_Read(uint8_t registerAddress, uint8_t *buffer, size_t bufferSize) {
	HAL_StatusTypeDef hStatus;

	if (bufferSize != 2) {
		return INA219_Status_NotSupported;
	}

	uint8_t bufferSwapped[2];

	hStatus = HAL_I2C_Mem_Read(&hi2c4, INA219_I2C_WRITE_ADDRESS, registerAddress, 1, bufferSwapped, bufferSize, 100);
	if (hStatus) {
		uint32_t errno = HAL_I2C_GetError(&hi2c4);

		if (errno & HAL_I2C_ERROR_AF) {
			return INA219_Status_I2CNack;
		} else if (errno & HAL_I2C_ERROR_TIMEOUT) {
			return INA219_Status_I2CTimeout;
		} else {
			return INA219_Status_I2CError;
		}
	}

	buffer[0] = bufferSwapped[1];
	buffer[1] = bufferSwapped[0];

	return INA219_Status_Ok;
}

INA219_Status INA219_PlatformSpecific_Write(uint8_t registerAddress, uint8_t *buffer, size_t bufferSize) {
	HAL_StatusTypeDef hStatus;

	if (bufferSize != 2) {
		return INA219_Status_NotSupported;
	}

	uint8_t bufferSwapped[2];
	bufferSwapped[0] = buffer[1];
	bufferSwapped[1] = buffer[0];

	hStatus = HAL_I2C_Mem_Write(&hi2c4, INA219_I2C_WRITE_ADDRESS, registerAddress, 1, bufferSwapped, bufferSize, 100);
	if (hStatus) {
		uint32_t errno = HAL_I2C_GetError(&hi2c4);

		if (errno & HAL_I2C_ERROR_AF) {
			return INA219_Status_I2CNack;
		} else if (errno & HAL_I2C_ERROR_TIMEOUT) {
			return INA219_Status_I2CTimeout;
		} else {
			return INA219_Status_I2CError;
		}
	}

	return INA219_Status_Ok;
}
