#include "MAX40080_PlatformSpecific.h"
#include "stm32l5xx_hal.h"

static I2C_HandleTypeDef i2c;
static int isInitialized = 0;

MAX40080_Status MAX40080_PlatformSpecific_Init() {
	HAL_StatusTypeDef hStatus;

	if (isInitialized) {
		return MAX40080_Status_InvalidOperation;
	}

	GPIO_InitTypeDef gpio;
	gpio.Pin = MAX40080_SCL_GPIO_PIN;
	gpio.Pull = GPIO_NOPULL;
	gpio.Speed = GPIO_SPEED_FREQ_LOW;
	gpio.Alternate = GPIO_AF4_I2C1;
	gpio.Mode = GPIO_MODE_OUTPUT_OD;

	HAL_GPIO_Init(MAX40080_SCL_GPIO, &gpio);
	for (int i = 0; i < 9; i++) {
		HAL_GPIO_WritePin(MAX40080_SCL_GPIO, MAX40080_SCL_GPIO_PIN, 0);
		HAL_Delay(1);
		HAL_GPIO_WritePin(MAX40080_SCL_GPIO, MAX40080_SCL_GPIO_PIN, 1);
		HAL_Delay(1);
	}

	gpio.Mode = GPIO_MODE_AF_OD;
	HAL_GPIO_Init(MAX40080_SCL_GPIO, &gpio);

	i2c.Instance = MAX40080_I2C_INSTANCE;
	i2c.Init.Timing = 0x40505681;
	i2c.Init.OwnAddress1 = 0;
	i2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
	i2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
	i2c.Init.OwnAddress2 = 0;
	i2c.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
	i2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
	i2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;

	hStatus = HAL_I2C_Init(&i2c);
	if (hStatus) {
		return MAX40080_Status_I2CError;
	}

	hStatus = HAL_I2CEx_ConfigAnalogFilter(&i2c, I2C_ANALOGFILTER_ENABLE);
	if (hStatus) {
		HAL_I2C_DeInit(&i2c);
		return MAX40080_Status_I2CError;
	}

	hStatus = HAL_I2CEx_ConfigDigitalFilter(&i2c, 0);
	if (hStatus) {
		HAL_I2C_DeInit(&i2c);
		return MAX40080_Status_I2CError;
	}

	isInitialized = 1;

	return MAX40080_Status_Ok;
}

MAX40080_Status MAX40080_PlatformSpecific_Deinit() {
	if (!isInitialized) {
		return MAX40080_Status_InvalidOperation;
	}

	HAL_I2C_DeInit(&i2c);

	isInitialized = 0;

	return MAX40080_Status_Ok;
}

MAX40080_Status MAX40080_PlatformSpecific_Read(uint8_t registerAddress, uint8_t *buffer, size_t bufferSize) {
	HAL_StatusTypeDef hStatus;

	hStatus = HAL_I2C_Mem_Read(&i2c, MAX40080_I2C_WRITE_ADDRESS, registerAddress, 1, buffer, bufferSize, 100);
	if (hStatus) {
		uint32_t errno = HAL_I2C_GetError(&i2c);

		if (errno & HAL_I2C_ERROR_AF) {
			return MAX40080_Status_I2CNack;
		} else if (errno & HAL_I2C_ERROR_TIMEOUT) {
			return MAX40080_Status_I2CTimeout;
		} else {
			return MAX40080_Status_I2CError;
		}
	}

	return MAX40080_Status_Ok;
}

MAX40080_Status MAX40080_PlatformSpecific_Write(uint8_t registerAddress, uint8_t *buffer, size_t bufferSize) {
	HAL_StatusTypeDef hStatus;

	hStatus = HAL_I2C_Mem_Write(&i2c, MAX40080_I2C_WRITE_ADDRESS, registerAddress, 1, buffer, bufferSize, 100);
	if (hStatus) {
		uint32_t errno = HAL_I2C_GetError(&i2c);

		if (errno & HAL_I2C_ERROR_AF) {
			return MAX40080_Status_I2CNack;
		} else if (errno & HAL_I2C_ERROR_TIMEOUT) {
			return MAX40080_Status_I2CTimeout;
		} else {
			return MAX40080_Status_I2CError;
		}
	}

	return MAX40080_Status_Ok;
}

MAX40080_Status MAX40080_PlatformSpecific_QuickCommand() {
	return MAX40080_Status_NotImplemented;
}

MAX40080_Status MAX40080_PlatformSpecific_SetI2CSpeed(MAX40080_I2CSpeed speed) {
	return MAX40080_Status_NotImplemented;
}
