#include "SevenSegment.h"

#include "stm32l5xx_hal.h"
#include <stdint.h>
#include <stdio.h>

static uint16_t displayLetters[DISPLAY_DIGITS];
static int actualDigit = 0;

static GPIO_TypeDef *digitSelectGpio[DISPLAY_DIGITS] = { GPIOB, GPIOB, GPIOB, GPIOC, GPIOB, GPIOE, GPIOB, GPIOC, GPIOC, GPIOC };
static uint16_t digitSelectPin[DISPLAY_DIGITS] = { 0x0001, 0x0002, 0x0004, 0x0008, 0x0010, 0x0004, 0x0040, 0x0001, 0x0002, 0x0002 };
static uint16_t digitPolarityLevel[DISPLAY_DIGITS] = { 1, 1, 1, 1, 1, 1, 1, 1, 0, 0 };

uint16_t digits[] = {
/* 0 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_F,
/* 1 */DISPA_SEG_B | DISPA_SEG_C,
/* 2 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_G | DISPA_SEG_D | DISPA_SEG_E,
/* 3 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_D | DISPA_SEG_G,
/* 4 */DISPA_SEG_F | DISPA_SEG_G | DISPA_SEG_B | DISPA_SEG_C,
/* 5 */DISPA_SEG_A | DISPA_SEG_F | DISPA_SEG_G | DISPA_SEG_C | DISPA_SEG_D,
/* 6 */DISPA_SEG_A | DISPA_SEG_F | DISPA_SEG_C | DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_G,
/* 7 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C,
/* 8 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_D | DISPA_SEG_E | DISPA_SEG_F | DISPA_SEG_G,
/* 9 */DISPA_SEG_A | DISPA_SEG_B | DISPA_SEG_C | DISPA_SEG_G | DISPA_SEG_F };

void SevenSegment_Init() {
	SevenSegment_Clear();
}

void SevenSegment_Clear() {
	for (int i = 0; i < DISPLAY_DIGITS; i++) {
		displayLetters[i] = 0;
	}
}

void SevenSegment_ClearA() {
	for (int i = 0; i < 8; i++) {
		displayLetters[i] = 0;
	}
}

void SevenSegment_ClearA_Green() {
	for (int i = 0; i < 4; i++) {
		displayLetters[i] = 0;
	}
}

void SevenSegment_ClearA_Red() {
	for (int i = 4; i < 8; i++) {
		displayLetters[i] = 0;
	}
}

void SevenSegment_ClearB() {
	for (int i = 8; i < 10; i++) {
		displayLetters[i] = 0;
	}
}

void SevenSegment_SetDigit(int digitIndex, uint16_t data) {
	if (digitIndex > 9) {
		__BKPT(0);
	}
	displayLetters[digitIndex] = data;
}

void SevenSegment_PrintFloat(int digit0Index, float num) {
	char buffer[32];
	snprintf(buffer, sizeof(buffer), "%f", num);

	char* bufferRd = buffer;
	for (int i = 0; i < 4; i++) {
		char letter = *bufferRd++;

		if (letter == '\0') {
			break;
		}

		int isDot = *bufferRd == '.';

		if (letter == '.') {
			letter = *bufferRd++;
		}

		if (letter >= '0' && letter <= '9') {
			uint16_t dispData = digits[letter - '0'];
			if (isDot) {
				dispData |= DISPA_SEG_DOT;
			}

			SevenSegment_SetDigit(digit0Index + i, dispData);
		} else if (letter == '-') {
			SevenSegment_SetDigit(digit0Index + i, DISPA_SEG_G);
		}
	}
}

void SevenSegment_RefreshDisplay() {
	int prevDigit = actualDigit;

	actualDigit++;
	if (actualDigit >= DISPLAY_DIGITS) {
		actualDigit = 0;
	}

	// disable prev digit
	HAL_GPIO_WritePin(digitSelectGpio[prevDigit], digitSelectPin[prevDigit], !digitPolarityLevel[prevDigit]);

	// update active segments
	HAL_GPIO_WritePin(GPIOD, displayLetters[actualDigit], 0);
	HAL_GPIO_WritePin(GPIOD, ~displayLetters[actualDigit], 1);

	// disable prev digit
	HAL_GPIO_WritePin(digitSelectGpio[actualDigit], digitSelectPin[actualDigit], digitPolarityLevel[actualDigit]);
}
