#include "StatusLed.h"
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os2.h"
#include "stm32l5xx_hal.h"

extern osMutexId_t sts_led1Handle;
extern osMutexId_t sts_led2Handle;

static osMutexId_t *ledMutexes[2] = { &sts_led1Handle, &sts_led2Handle };
static int ledColors[2] = { 0, 0 };
static int ledIsBlinking[2] = { 0, 0 };

void StatusLed_DriverThread(int channelNum, GPIO_TypeDef *redGpio, uint16_t redPin, GPIO_TypeDef *greenGpio, uint16_t greenPin) {
	while (1) {
		int isBlinking;
		int color;

		osMutexAcquire(*(ledMutexes[channelNum]), portMAX_DELAY);
		isBlinking = ledIsBlinking[channelNum];
		color = ledColors[channelNum];
		osMutexRelease(*(ledMutexes[channelNum]));

		if (isBlinking) {
			HAL_GPIO_WritePin(redGpio, redPin, 1);
			HAL_GPIO_WritePin(greenGpio, greenPin, 1);
			vTaskDelay(300);
		}
		if (color == 0) {
			HAL_GPIO_WritePin(redGpio, redPin, 1);
			HAL_GPIO_WritePin(greenGpio, greenPin, 1);
			vTaskDelay(5);
		} else {
			int count = 1;
			if (isBlinking) {
				if (color == 3) {
					count = 60;
				} else if (color == 2) {
					count = 300;
				} else if (color == 1) {
					count = 75;
				}
			}

			for (int i = 0; i < count; i++) {
				if (color & 1) {
					HAL_GPIO_WritePin(redGpio, redPin, 0);
					HAL_GPIO_WritePin(greenGpio, greenPin, 1);
					vTaskDelay(4);
				}
				if (color & 2) {
					HAL_GPIO_WritePin(redGpio, redPin, 1);
					HAL_GPIO_WritePin(greenGpio, greenPin, 0);
					vTaskDelay(1);
				}
			}
		}
	}
}

void StatusLed_SetState(int channelNum, int color, int isBlinking) {
	osMutexAcquire(*(ledMutexes[channelNum]), portMAX_DELAY);
	ledColors[channelNum] = color;
	ledIsBlinking[channelNum] = isBlinking;
	osMutexRelease(*(ledMutexes[channelNum]));
}

