#include "stm32l5xx_hal.h"
#include <string.h>
#include <stdio.h>

static UART_HandleTypeDef uart;

int UART_Init(void) {
	HAL_StatusTypeDef hStatus;

	__HAL_RCC_GPIOG_CLK_ENABLE();
	__HAL_RCC_LPUART1_CLK_ENABLE();
	HAL_PWREx_EnableVddIO2();

	GPIO_InitTypeDef gpio;
	gpio.Pin = GPIO_PIN_7 | GPIO_PIN_8;
	gpio.Mode = GPIO_MODE_AF_PP;
	gpio.Pull = GPIO_NOPULL;
	gpio.Speed = GPIO_SPEED_FREQ_LOW;
	gpio.Alternate = GPIO_AF8_LPUART1;

	HAL_GPIO_Init(GPIOG, &gpio);

	uart.Instance = LPUART1;
	uart.Init.BaudRate = 115200;
	uart.Init.WordLength = UART_WORDLENGTH_8B;
	uart.Init.StopBits = UART_STOPBITS_1;
	uart.Init.Parity = UART_PARITY_NONE;
	uart.Init.Mode = UART_MODE_TX;
	uart.Init.HwFlowCtl = UART_HWCONTROL_NONE;
	uart.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
	uart.Init.ClockPrescaler = UART_PRESCALER_DIV1;
	uart.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
	uart.FifoMode = UART_FIFOMODE_DISABLE;

	hStatus = HAL_UART_Init(&uart);
	if (hStatus != HAL_OK) {
		return -1;
	}

	hStatus = HAL_UARTEx_DisableFifoMode(&uart);
	if (hStatus != HAL_OK) {
		return -1;
	}

	return 0;
}

int UART_PrintString(char *string) {
	HAL_StatusTypeDef hStatus;

	size_t len = strlen(string);
	hStatus = HAL_UART_Transmit(&uart, (unsigned char*) string, len, 1000);
	if (hStatus) {
		return -1;
	}

	return 0;
}

int UART_PrintFormattedInteger(char *format, int number) {
	char buffer[32];
	int retval;

	retval = snprintf(buffer, sizeof(buffer), format, number);
	if (retval < 0 || retval >= sizeof(buffer)) {
		return -1;
	}

	return UART_PrintString(buffer);
}
